| Format for floating point numbers is:
| |man|tis|sa*|exp|  * - sign bit
| mantissa is unsigned, normalized to 1.fraction, 1. is not stored.
| exponent is offset 0x7f, values 0 and ff not allowed.
| This is IEEE format rotated to line up the exponent byte
|	Modified for DEC floating point format 12-20-82

			.data
			.text
			.globl  _fpmult,fpmult
			.globl  _fpdiv,fpdiv
			.globl  _fpplus,fpplus
			.globl  _fpminus,fpminus
			.globl  _fpfix,fpfix
			.globl  _fpsqrt,fpsqrt

| FPMULT computes d0*d1. Result in d0,
| d2 - partial products, d3 - partial sums, d4 - exponent holder
| d5 - CCR storage
fpmult:
_fpmult:	link    a6,#-20         | Save d1-d5
			moveml  #0x7c00,a6@-
			swap	d0				| align exponent byte
			rorl	#7,d0
			swap	d1				| align exponent byte
			rorl	#7,d1
			clrw    d5              |Zap the CC storage byte
			movw    d0,d2           | Move the first exponent
			beq     mdzerores       | 0*d0 = 0
			movw    d1,d3
			beq     mdzerores       | d0*0 = 0
| Now exponent bytes are in d2 and d3; compute exponent/sign into d4
			movw    d2,d4
			eorw    d3,d4           | So much for the sign bit
			btst    #8,d4           | If the sign will be negative,
			beq     mnotneg
			bset    #3,d5           | mark the CC byte appropriately.
mnotneg:	movb    #0x80,d4
			addb    d2,d4           | Now remove offsets
			subb    #0x80,d3        | from exponent bytes
			addb    d3,d4           | and add them
			bvs     mdovunder       | If overflow, you blew it (I think...)
			addb	#0x80,d4		| Put back the offset
			beq	mdunder				| and lose on fp underflow
| d4 is now 7 bits trash, sign bit, exponent. Now multiply mantissas.
			andw    #0xfe00,d0      | First, remove exponent sign bits
			andw    #0xfe00,d1
			bset    #0,d0           | Then put the 24th bit back in
			bset    #0,d1
			rorl    #1,d0           | and put it where it belongs
			rorl    #1,d1
			swap    d0              | Now compute d0h*d1l
			movw    d0,d3
			mulu    d1,d3
			swap    d0              | Now compute d1h*d0l
			swap    d1
			movw    d0,d2
			mulu    d1,d2
			addl    d2,d3           | d3 is now s3s2s100, carry in X
			clrw    d3
			roxlw   #1,d3
			swap    d3              | d3 is now 000*s3s2, carry in *
			swap    d0              | Now compute d0h*d1h into d0
			mulu    d1,d0
			addl    d3,d0           | d0 now all mantissa, carry in X
			bmi	mnobump				|Fraction already normalized
			roll	#2,d0			| not normalized, shift it
			subqb	#1,d4			| and bump the exponent
			beq	mdunder				| blowing up for bad values
			bra	mdfinal
mnobump:	roll	#1,d0			|Dump the 24th bit,
			cmpb	#0x7f,d4		| exponent may yet be too big
			beq	mdover				| it is; we lose
			bra	mdfinal

| FPDIV computes d0/d1. Result in d0,
| d2 - partial products, d3 - partial sums, d4 - exponent holder
| d5 - CCR storage
fpdiv:
_fpdiv:		link    a6,#-20         | Save d1-d5
			moveml  #0x7c00,a6@-
			swap	d0				| align exponent byte
			rorl	#7,d0
			swap	d1				| align exponent byte
			rorl	#7,d1
			clrb    d5              | Zap the CC storage byte
			movw    d1,d3           | Move the first exponent
			beq     mdover          | d0/0 = undefined, certainly wrong
			movw    d0,d2
			beq     mdzerores       | 0/d0 = 0
| Now exponent bytes are in d2 and d3; compute exponent/sign into d4
			movw    d2,d4
			eorw    d3,d4           | So much for the sign bit
			btst    #8,d4           | If the sign will be negative,
			beq     dnotneg
			bset    #3,d5           | mark the CC byte appropriately.
dnotneg:	movb    #0x80,d4
			addb    d2,d4           | Now remove offsets
			subb    #0x80,d3        | from exponent bytes
			subb    d3,d4           | and subtract them
			bvs     mdovunder		| no overflow, probably right
			addb	#0x80,d4		| Put back the offset
			beq	mdunder				| and lose on fp underflow
| d4 is now 7 bits trash, sign bit, exponent. Now divide mantissas.
			andw    #0xfe00,d0      | First, remove exponent sign bits
			andw    #0xfe00,d1
			addqb   #1,d0           | Then put the 24th bit back in
			addqb   #1,d1
			rorl    #2,d0           | and put it where it belongs
			rorl    #1,d1           | in each word
| Now do the divide ala Knuth vol. 2. (more or less)
			swap    d1              | Flop y to get at yh
			divu    d1,d0           | zh and remainder
			swap    d0              | Move quotient, align remainder
			movw    d0,d2           | Move remainder to d2
			swap    d2              | Move it over
			clrw    d2              | Zap its low bits
			divu    d1,d2           | & and compute rest of fraction
			movw    d2,d0           | Assemble fraction in d0
			movl    d0,d2           | and d2
			swap    d2              | Just to get at zh
			swap    d1              | and yl
			mulu    d1,d2           | zh yl
			swap    d1
			divu    d1,d2           | zh yl/yh
			andl    #0xffff,d2      | Zap remainder; quotient shifted OK
			subl    d2,d0           | Final fraction now in D0 (Yaaay!)
			bmi     dnonorm         | Already normalized? (it happens...)
			roll    #1,d0           | Not normalized, shift once
			subqb   #1,d4           | bump exponent(errors get caught later)
dnonorm:	roll    #1,d0           | Pitch the 24th bit again
			bra     mdfinal         | ok to return

| Code for handling exponent over/underflows
mdovunder:	bpl     mdunder         | Decide if it's over or underflow
mdover:		bset    #1,d5           | Overflow, set V bit only
			movb    #0x7e,d4        | fix the exponent
			movl    #0xfffffefe,d0  | ans. is max value, appropriately +/-
			bra     mdfinal         | now, return
mdunder:	orb     #0xa,d5         | It's undeflow, set V and N bits
mdzerores:	bset    #2,d5           | and the Z bit (we're returning 0)
			clrl    d0              | mantissa is 0
			clrw    d4              | as is exponent and sign

| Shared code for final cleaning up
mdfinal:	andw    #0xfe00,d0      | stomp on the exponent field
			andw    #0x1ff,d4       | and the trash bits
			orw     d4,d0           | then put them together
			movw    d5,cc           | load appropriate values into flags
			moveml  a6@+,#0x003e    | restore registers
			roll	#7,d0			| Put it back in DEC mode
			swap	d0
			unlk    a6
			rts                     | and return.


|FPPLUS and FPMINUS do floating point +/- of D0 and D1.
|They use D2 for exponent calculation.
fpminus:
_fpminus:	link    a6,#-8          | Save d1 & d2
			moveml  #0x6000,a6@-
			bchg    #31,d1          | d0-d1 = d0+(-d1)
			bra     fpadd           | go do the addition
fpplus:
_fpplus:	link    a6,#-8          | Save d1 & d2
			moveml  #0x6000,a6@-
fpadd:		swap	d0				| align exponent byte
			rorl	#7,d0
			swap	d1				| align exponent byte
			rorl	#7,d1
			clrl    d2              | exponent storage
			tstb    d1              | Are we adding a 0?
			beq     pmshort         | d0+0 = d0
			tstb    d0              | Are we adding a 0?
			bne     pmnotzero       | 0+d1 = d1
			movl    d1,d0
			bra     pmshort         | Now go and set CC
pmnotzero:	subb    #0x80,d0        | remove exponent offsets
			subb    #0x80,d1        | and see if we have to swap
			cmpb    d0,d1
			ble     pmnoswap
			exg     d0,d1           | Yes, we do
pmnoswap:	movb    d0,d2           | Now, how much do we have to shift?
			subb    d1,d2
			cmpb    #24,d2          | Is it too much?
			blt     pmnottoobig
			addb    #0x80,d0        | Yes, result is in d0
			bra     pmshort
pmnottoobig:swap    d2              | Set up final exponent here
			movb    d0,d2           | in other half of d2
			movb    #1,d0           | Put back 24th bits
			movb    d0,d1
			rorl    #3,d0           | Shift big one over
			bclr    #5,d0           | and test its sign
			beq     pmnoneg0
			negl    d0              | negating it if necessary
pmnoneg0:	rorl    #3,d1           | Shift d1 the same way
			bclr    #5,d1
			beq     pmnoneg1
			negl    d1
pmnoneg1:	swap    d2              | Get the shift # back for a second
			asrl    d2,d1           | and shift D1 over appropriately
			swap    d2              | then get the exponent handy
			addl    d1,d0           | and add them (Yay!)
| There can't be any overflow: just check signs and normalize
			bpl     pmnoneg2        | Is the result negative?
			negl    d0              | Yes, negate mantissa
			bset    #8,d2           | and set the sign bit
pmnoneg2:	tstl    d0              | Test d0 again
			bne     pmnotzero2      | Do we have zero?
			clrl    d0              | let pmdone set Z bit
			bra     pmdone          | and go there
pmnotzero2:	asll    #2,d0           | otherwise shift over the mantissa
			bcc     pmshift         | do we have to shift it any more?
			addqb   #1,d2           | No, bump exponent once
			bvs     pmover          | and leave now if we blew it
			bra     pmnorm          | otherwise, go do final assembly
pmshift:	asll    #1,d0           | Shift once more
			bcs     pmnorm          | and leave if we're done
			subqb   #1,d2           | otherwise, bump the exponent
			bvc     pmshift         | and loop back if we didn't overflow
pmover:		movl    #0xfffffefe,d0  | Overflow; send back biggest number
			orb     #2,d1           | and set V flag
			bra     pmdone          | and go finish up
pmunder:	clrl    d0              | we did; return 0
			moveq   #0xa,d1         | and CC is ZV
			bra     pmdone
pmnorm:		andw    #0xfe00,d0      | Done, zap exponent byte and sign bit
			addb    #0x80,d2        | re-offset exponent
			orw     d2,d0           | reassemble the number
pmshort:	clrb    d1              | and get ready to set CC bits
			tstb    d0
			bne     pmnot0          | Do we have 0?
			orb     #0x4,d1         | Yes, set Z bit
			bra     pmdone          | and finish up
pmnot0:		btst    #8,d0           | check the sign bit
			beq     pmdone          | leave if positive
			orb     #0x8,d1         | Set CC N bit if negative
pmdone:		movw    d1,cc           | Load up CCR
			moveml  a6@+,#0x0006    | Restore registers
			roll	#7,d0			| Put it back in DEC mode
			swap	d0
			unlk    a6
			rts                     | and return

|FPFIX returns the smallest integer not larger than the floating number in d0.
fpfix:
_fpfix:		movl    d1,sp@-         | save d1
			swap	d0				| Align the exponent
			rorl	#7,d0
			beq		fnoflop			| If we have floating 0, handle it
			movw    d0,d1           | snag the exponent and sign
			andw    #0xfe00,d0      | then clear them out of d0
			addqb   #1,d0           | put back the 24th bit
			rorl    #2,d0           | shift over
			addb    #0x61,d1        | set up shift count in d1 (0x61 is
			negb    d1              | a magic number that works here)
			bge     fnottoobig      | negative shift count, it's too big
			tstl    d0              | get appropriate sign into CC
			orb     #2,cc           | set overflow CC
			bra     fdone           | and return
fnottoobig:	cmpb    #30,d1          | is it too small?
			ble     fnottoosmall
			clrl    d0              | yes, return 0
			orb     #2,cc           | set overflow CC
			bra     fdone           | and return
fnottoosmall:asrl    d1,d0			| no, shift it over appropriately
fdone:		btst    #8,d1           | and see if we have to negate it
			beq     fnoflop
			negl    d0              | we do
fnoflop:	movl    sp@+,d1         | restore d1
			rts                     | and return

| FPSQRT takes the square root of the number in d0
| Algorithm:
| d0= 1.mant * 2**exp
| To find sqrt(ms), take first approximation to be y0= 1.mant * 2**(exp/2),
| then do Newton-Raphson yn+1= 1/2*(yn + ms/yn) an appropriate number of times
fpsqrt:
_fpsqrt:	link    a6,#-16         | save registers d1-d4
			moveml  #0x7800,a6@-    
			movl    d0,d2           | copy the number (ms) in d2
			beq     sqdone          | if it's 0, sqrt is 0
			swap	d0				| align exponent byte
			rorl	#7,d0
			subb    #0x80,d0        | remove offset from exponent
			asrb    #1,d0           | "divide" exponent by 2
			addb    #0x80,d0        | and put offset back
			roll	#7,d0
			swap	d0				| align exponent byte
			moveq   #3,d4           | d4 counts iterations
sqnewton:	movl    d0,d3           | save yn in d3
			movl    d2,d0           | and prepare
			movl    d3,d1           | to compute
			jsr     fpdiv           | ms/yn
			movl    d3,d1           | Get yn again and
			jsr     fpplus          | compute ms/yn + yn
			roll	#1,d0			|align exponent byte
			subb	#0x01000000,d0	| 1/2 (ms/yn + yn) = yn+1
			rorl	#1,d0			|Re-align exponent byte
			dbf     d4,sqnewton     | loop back if necessary
sqdone:		moveml  a6@+,#0x001e    | restore registers
			unlk    a6
			rts                     | and return
