| Displacements in object block
			IX = 0
			IY = 2
			X  = 4
			Y  = 8
			XVEL = 12
			YVEL = 16
			XT = 20
			YT = 24
			FLAGS = 28
| Gravitational constant (set in main program)
			.globl	_G
			
| Rollover constants (floating and fixed)
	XMAX	= 0x44f80000	| actually 496
	YMAX	= 0x44c00000	| actually 384
	XADJUST = 0x45780000	| actually 992
	YADJUST = 0x45400000	| actually 768
	IXMAX	=	496
	IXADJUST =	992
	IYMAX	=	384
	IYADJUST =	768

			.globl  _orbit
			.text
_orbit: 	link    a6,#0
			moveml  #0xff04,sp@-    | Push all d registers and a5
			movl    a6@(8),a5       | Get address of object into a5
| Now for serious orbital calcualtion
| First compute super constant G / R ** 3
			movl    a5@(X),d0
			movl    d0,d1
			jsr     fpmult          | X ** 2
			movl    d0,d2           | in d2
			movl    a5@(Y),d0
			movl    d0,d1
			jsr     fpmult          | Y ** 2
			movl    d2,d1           | retrieve X ** 2
			jsr     fpplus          | R ** 2
			movl    d0,d7           | saved in d7
			movl    d0,d1           | copied
			jsr     fpsqrt          | R
			jsr     fpmult          | R ** 3
			movl    d0,d6           | saved in d6
			movl    d0,d1           | and moved
			movl    _G,d0
			jsr     fpdiv           | G / R ** 3
			movl    d0,d5           | saved in d5
| Now use constant to nudge velocities and hence positions
| First X
			movl    a5@(X),d1
			jsr     fpmult          | X * (G / R ** 3)
			movl    a5@(XVEL),d1
			jsr     fpplus          | new XVEL
			btst    #7,a5@(FLAGS)   | engine on?
			beq     xnoton
			movl    a5@(XT),d1      | Yes, bump velocity
			jsr     fpplus          | XVEL
xnoton:		movl    d0,a5@(XVEL)    | replaced
			movl    a5@(X),d1
			jsr     fpplus          | new X
			movl    d0,a5@(X)       | X replaced
| Then Y
			movl    d5,d0           | restore (G / R ** 3)
			movl    a5@(Y),d1
			jsr     fpmult          | Y * (G / R ** 3)
			movl    a5@(YVEL),d1
			jsr     fpplus          | new YVEL
			btst    #7,a5@(FLAGS)   | engine on?
			beq     ynoton
			movl    a5@(YT),d1      | Yes, bump velocity
			jsr     fpplus
ynoton:		movl    d0,a5@(YVEL)    | replaced
			movl    a5@(Y),d1
			jsr     fpplus          | new Y
			movl    d0,a5@(Y)       | Y replaced
| Then integerize results and store them in block
			jsr     fpfix           | First Y
|Now check to see if we need to rollover
			movl	d0,d2			| Hide a copy in d2
			bpl		ylt01			| Do we need to negate Y?
			negl	d0				| Yes.
ylt01:		cmpl	#IYMAX,d0		| Is |y| > IYMAX?
			ble		yok				| No, go store it
			movl	a5@(Y),d0		| Yes, move everything up for rollover
			movl	#YADJUST,d1
			tstl	d2				| Now, what was Y's sign?
			bmi		ylt02
			jsr		fpminus			| Oh yeah... positive: replace Y
			movl	d0,a5@(Y)
			subl	#IYADJUST,d2	
			bgt		relativity		| If Y is still positive, go Einstein them
			bra		yok				| No, y is finally OK

ylt02:		jsr		fpplus			| Actually Y was negative...
			movl	d0,a5@(Y)		| replace Y
			addl	#IYADJUST,d2	
			blt		relativity		| If Y is still negative, go Einstein them
yok:		movw    d2,a5@(IY)		| (Finally!) Replace IY
			movl    a5@(X),d0       | Then X
			jsr     fpfix
			movl	d0,d2			| Hide a copy in d2
			bpl		xlt01			| Do we need to negate X?
			negl	d0				| Yes.
xlt01:		cmpl	#IXMAX,d0		| Is |X| > IXMAX?
			ble		xok				| No, go store it
			movl	a5@(X),d0		| Yes, move everything up for rollover
			movl	#XADJUST,d1
			tstl	d2				| Now, what was X's sign?
			bmi		xlt02
			jsr		fpminus			| Oh yeah... positive: replace X
			movl	d0,a5@(X)
			subl	#IXADJUST,d2	
			bgt		relativity		| If X is still positive, go Einstein them
			bra		xok				| No, x is finally OK

xlt02:		jsr		fpplus			| Actually X was negative...
			movl	d0,a5@(X)		| replace X
			addl	#IXADJUST,d2	
			blt		relativity		| If X is still negative, go Einstein them
xok:		movw    d2,a5@(IX)		| (Finally!) Replace IX
| Done, restore registers and return
			moveml  sp@+,#0x20ff
			unlk    a6
			rts
relativity:			| This ship is exceeding C (~ 800 units/tick)
					| So throw it to the center of the sun!
			clrw	a5@(IX)
			clrw	a5@(IY)
			clrl	a5@(X)
			clrl	a5@(Y)
			clrl	a5@(XVEL)
			clrl	a5@(YVEL)
			moveml  sp@+,#0x20ff
			unlk    a6
			rts
