	/* the gravitational constant */
#define GRAVITY 	-3000.0
	/* engine thrust */
#define ETHRUST 	.1
#define BDELV		3.
#define BOUNCE		3.
#define ELASTICITY	0.8
#define SHIPMASS	5.0
#define MAXSHIPS	4
#define BOMBSPERSHIP 	8
#define MAXBOMBS	(BOMBSPERSHIP * MAXSHIPS)
#define INITR		260.
#define INITV		1.5
#define BDELP		23.
#define PI		3.14159
	/* offsets for ship-sized things */
#define SXOFF		(511-15)
#define SYOFF		(623-15)
	/* offsets for bomb-sized things */
#define BXOFF		(511-7)
#define BYOFF		(623-7)
#define ANGLES		256
#define TURNRATE 	5
	/* the minimum altitude allowed */
#define MINALT 		8

/* assorted random clock limits */
	 /* time it takes a ship to die after shot */
#define SDIETIME	32 
	/* time between bomb shots */
#define SCOOLTIME	20
	  /* time it takes a bomb to die after hitting */
#define BDIETIME	20
	 /* total lifetime of a bomb */
#define BLIFETIME      512
	  /* lifetime of things that hit the sun */
#define FLASHTIME	10
	  /* time survivor must wait until a new game */
#define TWIDDLETIME	64

#define INITFILE	"/usr/games/lib/orbit.tables"
#define SHIPDIRECTORY	"/usr/games/lib/orbit"

typedef short sframe[62];	/* convenient typedef for movies and ships */
				/* it's the size of one ship frame */
typedef short bframe[15];	/* convenient typedef for bombs */
				/* it's the size of one bomb frame */

/* a structure to hold all the pre-computed tables in one file */
struct {
	sframe	sun[ANGLES];	/* sun movie rasters */
	sframe	boom[ANGLES];	/* explosion movie rasters */
	sframe	flame[ANGLES];	/* flame movie rasters */
	bframe	bomb[ANGLES];	/* bomb movie rasters */
	float 	thrust[ANGLES];	/* table of thrust values */
	float	bdelp[ANGLES];	/* table of bomb position offsets */
	float	bdelv[ANGLES];	/* table of bomb velocity offsets */
	} t;

/* structures for ship and bomb specific parts of a mass */
struct ship_specific
	{
	sframe	*frame;		/* points to current ship frame */
	int		bombset;	/* bit vector of possible bombs */
	short	score;		/* what else? */
	char	roundscore;	/* score for this round */
	unsigned char bombcount;	/* bombs left in internal rack */
	};
struct bomb_specific
	{
	bframe	*frame;		/* points to current bomb frame */
	char	radius;		/* size of bomb ((un)exploded) */
	unsigned char
			shipset,	/* bit vector of possible ships */
			owner;		/* number of bomb's owner */
	};

/* nothing above engineon in this structure can be moved without modifying
	sorbit.s; it has the offsets for those fields and that bit burned into it */
struct mass
	{
	short ix,iy;           /* integer coordinates */
	float x,y,             /* floating coodinates */
	xvel,yvel,             /* floating velocity */
	xt,yt;                 /* floating thrust */
	unsigned char
		engineon:    1,    /* thrust key is down */
		well:        1,    /* cruising, not exploding */
		alive:       1,    /* not dead (naturally!) */
		shooting:    1,    /* shoot key is down */
		cooling:     1,    /* shot clock is counting */
		suicide:     1;    /* ship died on own bomb this round */
	unsigned char
		theta,             /* angle ship is facing (0-255, naturally) */
		ton;               /* direction and speed if ship/bomb is turning */
	short scix,sciy;       /* screen coordinates */
	short clock;           /* general_purpose clock counter */
	union
		{
		struct ship_specific	 s;	/* ship specific data */
		struct bomb_specific	 b;	/* bomb specific data */
		}    u;
	};

/* a shorter way to wound a ship */
#define wound(m,c) (m->well=0, m->clock= c, m->u.s.frame = (sframe *) t.boom)

/* Paranoid mover; checks for overlapping blocks and cuts them up */

#define parmove(sx,sy,dx,dy,w,h,stuff)\
	{register int delx,dely;\
	delx= dx-sx; dely= dy-sy;\
	ras_put(dx,dy,h,w,stuff,GXcopy);\
	GXfunction= GXclear;\
	if (abs(delx) > w || abs(dely) > h) /* no chance of overlap */\
		ras_write(w,h,sx,sy);\
	else\
/* we have an overlap, erase whatever didn't get overwritten */\
		{\
		if(dely > 0)\
			ras_write(w,dely,sx,sy);\
		else if (dely < 0)\
			ras_write(w,-dely,sx,sy+h+dely);\
		if(delx > 0)\
			ras_write(delx,h,sx,sy);\
		else if (delx < 0)\
			ras_write(-delx,h,sx+w+delx,sy);\
		}\
	}

#define MINIMUM(x, y) ((x)>(y) ? (y) : (x))
